@@ -1,5 +1,5 @@
 # This file is part of Epoptes, https://epoptes.org
-# Copyright 2010-2020 the Epoptes team, see AUTHORS.
+# Copyright 2010-2023 the Epoptes team, see AUTHORS.
 # SPDX-License-Identifier: GPL-3.0-or-later
 """
 Configuration file parser and other default configuration variables.
@@ -174,6 +174,8 @@ system.setdefault('PORT', 789)
 system.setdefault('SOCKET_GROUP', 'epoptes')
 system.setdefault('DIR', '/run/epoptes')
 system.setdefault('THUMBSHOT_MS', 5000)
+# The epoptes SERVER key is used for SSH socket forwarding
+system.setdefault('SERVER', 'server')
 # Allow running unencrypted, for clients with very low RAM.
 try:
     if os.path.getsize('/etc/epoptes/server.crt') == 0:
